@@ -31,8 +31,7 @@
 
 #include "gkr-pam.h"
 
-#include "library/gnome-keyring-result.h"
-#include "library/gnome-keyring-opcodes.h"
+#include "daemon/control/gkd-control-codes.h"
 
 #include <security/pam_appl.h>
 #include <security/pam_modules.h>
@@ -67,10 +66,8 @@ enum {
 	ARG_USE_AUTHTOK	        = 1 << 2
 };
 
-#define LOGIN_KEYRING		"login"
-
-#define ENV_SOCKET 		"GNOME_KEYRING_SOCKET"
-#define ENV_PID    		"GNOME_KEYRING_PID"
+#define ENV_CONTROL             "GNOME_KEYRING_CONTROL"
+#define ENV_PID                 "GNOME_KEYRING_PID"
 
 /* read & write ends of a pipe */
 #define  READ_END   0
@@ -230,7 +227,7 @@ evaluate_inlist (const char *needle, const char *haystack)
 	const char *remaining;
 
 	if (!needle)
-		return PAM_BAD_ITEM;
+		return 0;
 
 	remaining = haystack;
 
@@ -244,7 +241,7 @@ evaluate_inlist (const char *needle, const char *haystack)
 			item += strlen (needle);
 			/* is item really needle? */
 			if (*item == '\0' || *item == ',')
-                                return PAM_SUCCESS;
+                                return 1;
 		}
 
                 remaining = strchr (item, ',');
@@ -255,7 +252,7 @@ evaluate_inlist (const char *needle, const char *haystack)
 		++remaining;
         }
 
-        return PAM_BAD_ITEM;
+        return 0;
 }
 
 /* -----------------------------------------------------------------------------
@@ -578,15 +575,15 @@ static int
 start_daemon_if_necessary (pam_handle_t *ph, struct passwd *pwd, 
                            const char *password, int* started)
 {
-	const char *socket;
+	const char *control;
 	int ret;
-	
+
 	*started = 0;
-	
+
 	/* See if it's already running, and transfer env variables */
-	socket = get_any_env (ph, ENV_SOCKET);
-	if (socket) {
-		ret = setup_pam_env (ph, ENV_SOCKET, socket);
+	control = get_any_env (ph, ENV_CONTROL);
+	if (control) {
+		ret = setup_pam_env (ph, ENV_CONTROL, control);
 		if (ret != PAM_SUCCESS) {
 			syslog (GKR_LOG_ERR, "gkr-pam: couldn't set environment variables: %s",
 			        pam_strerror (ph, ret));
@@ -644,70 +641,37 @@ done:
 	return PAM_SUCCESS;
 }
 
-static int
-create_keyring (pam_handle_t *ph, struct passwd *pwd, const char *password)
-{
-	const char *socket;
-	GnomeKeyringResult res;
-	const char *argv[2];
-	
-	assert (pwd);
-	assert (password);
-	
-	socket = get_any_env (ph, ENV_SOCKET);
-	if (!socket) {
-		syslog (GKR_LOG_WARN, "gkr-pam: couldn't create '%s' keyring: %s", 
-		        LOGIN_KEYRING, "gnome-keyring-daemon is not running");
-		return PAM_SERVICE_ERR;
-	}
-	
-	argv[0] = LOGIN_KEYRING;
-	argv[1] = password;
-	
-	res = gkr_pam_client_run_operation (pwd, socket, GNOME_KEYRING_OP_CREATE_KEYRING, 2, argv);
-	if (res != GNOME_KEYRING_RESULT_OK) {
-		syslog (GKR_LOG_ERR, "gkr-pam: couldn't create '%s' keyring: %d", LOGIN_KEYRING, res);
-		return PAM_SERVICE_ERR;
-	}
-	
-	
-	syslog (GKR_LOG_NOTICE, "gkr-pam: created '%s' keyring", LOGIN_KEYRING);
-	return PAM_SUCCESS; 
-}
-
 static int
 unlock_keyring (pam_handle_t *ph, struct passwd *pwd, const char *password)
 {
-	const char *socket;
-	GnomeKeyringResult res;
+	const char *control;
+	int res;
 	const char *argv[2];
 	
 	assert (pwd);
 	assert (password);
-	
-	socket = get_any_env (ph, ENV_SOCKET);
-	if (!socket) {
-		syslog (GKR_LOG_WARN, "gkr-pam: couldn't unlock '%s' keyring: %s", 
-		        LOGIN_KEYRING, "gnome-keyring-daemon is not running");
+
+	control = get_any_env (ph, ENV_CONTROL);
+	if (!control) {
+		syslog (GKR_LOG_WARN, "gkr-pam: couldn't unlock login keyring: %s",
+		        "gnome-keyring-daemon is not running");
 		return PAM_SERVICE_ERR;
 	}
 	
-	argv[0] = LOGIN_KEYRING;
-	argv[1] = password;
-	
-	res = gkr_pam_client_run_operation (pwd, socket, GNOME_KEYRING_OP_UNLOCK_KEYRING, 2, argv);
+	argv[0] = password;
 
-	/* 'login' keyring doesn't exist, create it */
-	if (res == GNOME_KEYRING_RESULT_NO_SUCH_KEYRING) {
-		return create_keyring (ph, pwd, password);
+	res = gkr_pam_client_run_operation (pwd, control, GKD_CONTROL_OP_UNLOCK, 1, argv);
 
 	/* An error unlocking */
-	} else if (res != GNOME_KEYRING_RESULT_OK) {
-		syslog (GKR_LOG_ERR, "gkr-pam: couldn't unlock '%s' keyring: %d", LOGIN_KEYRING, res);
+	if (res == GKD_CONTROL_RESULT_DENIED) {
+		syslog (GKR_LOG_ERR, "gkr-pam: the password for the login keyring was invalid.");
 		return PAM_SERVICE_ERR;
-	} 	
-		
-	syslog (GKR_LOG_INFO, "gkr-pam: unlocked '%s' keyring", LOGIN_KEYRING);
+	} else if (res != GKD_CONTROL_RESULT_OK) {
+		syslog (GKR_LOG_ERR, "gkr-pam: couldn't unlock the login keyring.");
+		return PAM_SERVICE_ERR;
+	}
+
+	syslog (GKR_LOG_INFO, "gkr-pam: unlocked login keyring");
 	return PAM_SUCCESS;
 }
 
@@ -715,39 +679,36 @@ static int
 change_keyring_password (pam_handle_t *ph, struct passwd *pwd, 
                          const char *password, const char *original)
 {
-	const char *socket;
-	GnomeKeyringResult res;
+	const char *control;
 	const char *argv[3];
-	
+	int res;
+
 	assert (pwd);
 	assert (password);
 	assert (original);
-	
-	socket = get_any_env (ph, ENV_SOCKET);
-	if (!socket) {
-		syslog (GKR_LOG_WARN, "gkr-pam: couldn't change password on '%s' keyring: %s", 
-		        LOGIN_KEYRING, "gnome-keyring-daemon is not running");
+
+	control = get_any_env (ph, ENV_CONTROL);
+	if (!control) {
+		syslog (GKR_LOG_WARN, "gkr-pam: couldn't change password on login keyring: %s",
+		        "gnome-keyring-daemon is not running");
 		return PAM_SERVICE_ERR;
 	}
 	
-	argv[0] = LOGIN_KEYRING;
-	argv[1] = original;
-	argv[2] = password;	
+	argv[0] = original;
+	argv[1] = password;
 	
-	res = gkr_pam_client_run_operation (pwd, socket, GNOME_KEYRING_OP_CHANGE_KEYRING_PASSWORD, 3, argv);
+	res = gkr_pam_client_run_operation (pwd, control, GKD_CONTROL_OP_CHANGE, 2, argv);
 
 	/* No keyring, not an error. Will be created at initial authenticate. */
-	if (res == GNOME_KEYRING_RESULT_NO_SUCH_KEYRING) {
-		syslog (GKR_LOG_INFO, "gkr-pam: '%s' keyring does not exist, not changing password", LOGIN_KEYRING);
-		return PAM_SUCCESS;
-		
-	/* An error occured unlocking */
-	} else if (res != GNOME_KEYRING_RESULT_OK) {
-		syslog (GKR_LOG_ERR, "gkr-pam: couldn't change password for '%s' keyring: %d", LOGIN_KEYRING, res);
+	if (res == GKD_CONTROL_RESULT_DENIED) {
+		syslog (GKR_LOG_ERR, "gkr-pam: couldn't change password for the login keyring: the passwords didn't match.");
+		return PAM_SERVICE_ERR;
+	} else if (res != GKD_CONTROL_RESULT_OK) {
+		syslog (GKR_LOG_ERR, "gkr-pam: couldn't change password for the login keyring.");
 		return PAM_SERVICE_ERR;
 	}
-	
-	syslog (GKR_LOG_NOTICE, "gkr-pam: changed password for '%s' keyring", LOGIN_KEYRING);
+
+	syslog (GKR_LOG_NOTICE, "gkr-pam: changed password for login keyring");
 	return PAM_SUCCESS;
 }
  
@@ -821,7 +782,7 @@ parse_args (pam_handle_t *ph, int argc, const char **argv)
 
 		} else if (strncmp (argv[i], "only_if=", only_if_len) == 0) {
 			const char *value = argv[i] + only_if_len;
-			if (evaluate_inlist (svc, value) != PAM_SUCCESS)
+			if (!evaluate_inlist (svc, value))
 				args |= ARG_IGNORE_SERVICE;
 
 		} else if (strcmp (argv[i], "use_authtok") == 0) {
@@ -841,7 +802,7 @@ pam_sm_authenticate (pam_handle_t *ph, int unused, int argc, const char **argv)
 {
 	struct passwd *pwd;
 	const char *user, *password;
-	const char *socket;
+	const char *control;
 	int started_daemon;
 	uint args;
 	int ret;
@@ -884,11 +845,11 @@ pam_sm_authenticate (pam_handle_t *ph, int unused, int argc, const char **argv)
 		if (ret != PAM_SUCCESS)
 			return ret;
 	}
-	
-	socket = get_any_env (ph, ENV_SOCKET);
+
+	control = get_any_env (ph, ENV_CONTROL);
 
 	/* If gnome keyring is running, then unlock now */
-	if (socket) {
+	if (control) {
 		/* If we started the daemon, its already unlocked, since we passed the password */
 		if (!started_daemon) {
 			ret = unlock_keyring (ph, pwd, password);
@@ -958,7 +919,7 @@ pam_sm_open_session (pam_handle_t *ph, int flags, int argc, const char **argv)
 	}
 
 	/* If gnome keyring is running, but we didn't start it here, then unlock now */
-	if (get_any_env (ph, ENV_SOCKET) != NULL) {
+	if (get_any_env (ph, ENV_CONTROL) != NULL) {
 		if (!started_daemon && password != NULL) {
 			if (unlock_keyring (ph, pwd, password) != PAM_SUCCESS)
 				return PAM_SERVICE_ERR;
@@ -1032,8 +993,8 @@ pam_chauthtok_update (pam_handle_t *ph, struct passwd *pwd, uint args)
 	
 	ret = pam_get_item (ph, PAM_OLDAUTHTOK, (const void**)&original);
 	if (ret != PAM_SUCCESS || original == NULL) {
-		syslog (GKR_LOG_WARN, "gkr-pam: couldn't update the '%s' keyring password: %s", 
-		        LOGIN_KEYRING, "no old password was entered");
+		syslog (GKR_LOG_WARN, "gkr-pam: couldn't update the login keyring password: %s",
+		        "no old password was entered");
 		return PAM_IGNORE;
 	}
 		
